@@ -1,24 +1,3 @@
-getDataTypeByNiftiCode <- function (code)
-{
-    typeIndex <- which(.Nifti$datatypes$codes == code)
-    if (length(typeIndex) != 1)
-        return (NULL)
-    else
-        return (list(type=.Nifti$datatypes$rTypes[typeIndex], size=.Nifti$datatypes$sizes[typeIndex], isSigned=.Nifti$datatypes$isSigned[typeIndex]))
-}
-
-getNiftiCodeForDataType <- function (datatype)
-{
-    if (length(datatype) == 0)
-        return (NULL)
-    
-    typeIndex <- which((.Nifti$datatypes$rTypes == datatype$type) & (.Nifti$datatypes$sizes == datatype$size) & (.Nifti$datatypes$isSigned == datatype$isSigned))
-    if (length(typeIndex) != 1)
-        return (NULL)
-    else
-        return (.Nifti$datatypes$codes[typeIndex])
-}
-
 hasNiftiMagicString <- function (fileName)
 {
     connection <- gzfile(fileName, "rb")
@@ -42,27 +21,9 @@ readNifti <- function (fileNames)
         else if (qformCode > 0)
         {
             report(OL$Debug, "Using qform (code ", qformCode, ") for origin")
-            matrix <- diag(4)
+            matrix <- quaternionToXform(quaternionParams[1:3])
             matrix[1:3,4] <- quaternionParams[4:6]
             
-            quaternionSumOfSquares <- sum(quaternionParams[1:3]^2)
-            if (equivalent(quaternionSumOfSquares, 1, tolerance=1e-6))
-                q <- c(0, quaternionParams[1:3])
-            else if (quaternionSumOfSquares > 1)
-                report(OL$Error, "Quaternion parameters are invalid")
-            else
-                q <- c(sqrt(1 - quaternionSumOfSquares), quaternionParams[1:3])
-
-            matrix[1:3,1:3] <- c(  q[1]*q[1] +   q[2]*q[2] - q[3]*q[3] - q[4]*q[4],
-                                 2*q[2]*q[3] + 2*q[1]*q[4],
-                                 2*q[2]*q[4] - 2*q[1]*q[3],
-                                 2*q[2]*q[3] - 2*q[1]*q[4],
-                                   q[1]*q[1] +   q[3]*q[3] - q[2]*q[2] - q[4]*q[4],
-                                 2*q[3]*q[4] + 2*q[1]*q[2],
-                                 2*q[2]*q[4] + 2*q[1]*q[3],
-                                 2*q[3]*q[4] - 2*q[1]*q[2],
-                                   q[1]*q[1] +   q[4]*q[4] - q[3]*q[3] - q[2]*q[2])
-
             # The qfactor should be stored as 1 or -1, but the NIfTI standard says
             # 0 should be treated as 1; this does that (the 0.1 is arbitrary)
             qfactor <- sign(voxelDims[1] + 0.1)
@@ -142,9 +103,10 @@ readNifti <- function (fileNames)
     
     xformMatrix <- getXformMatrix()
     
-    datatype <- getDataTypeByNiftiCode(typeCode)
-    if (is.null(datatype))
+    typeIndex <- which(.Nifti$datatypes$codes == typeCode)
+    if (length(typeIndex) != 1)
         report(OL$Error, "Data type of file ", fileNames$imageFile, " (", typeCode, ") is not supported")
+    datatype <- list(code=typeCode, type=.Nifti$datatypes$rTypes[typeIndex], size=.Nifti$datatypes$sizes[typeIndex], isSigned=.Nifti$datatypes$isSigned[typeIndex])
     
     # We're only interested in the bottom 5 bits (spatial and temporal units)
     spatialUnitCode <- packBits(intToBits(unitCode) & intToBits(7), "integer")
@@ -155,9 +117,9 @@ readNifti <- function (fileNames)
     
     dimsToKeep <- 1:max(which(dims > 1))
     
-    imageMetadata <- list(imageDims=dims[dimsToKeep], voxelDims=voxelDims[dimsToKeep+1], voxelUnit=voxelUnit, source=fileNames$fileStem, datatype=datatype, tags=list())
+    imageMetadata <- list(imageDims=dims[dimsToKeep], voxelDims=voxelDims[dimsToKeep+1], voxelUnit=voxelUnit, source=fileNames$fileStem, tags=list())
     
-    storageMetadata <- list(dataOffset=dataOffset, dataScalingSlope=slopeAndIntercept[1], dataScalingIntercept=slopeAndIntercept[2], xformMatrix=xformMatrix, endian=endian)
+    storageMetadata <- list(dataOffset=dataOffset, dataScalingSlope=slopeAndIntercept[1], dataScalingIntercept=slopeAndIntercept[2], xformMatrix=xformMatrix, datatype=datatype, endian=endian)
     
     invisible (list(imageMetadata=imageMetadata, storageMetadata=storageMetadata))
 }
@@ -167,42 +129,45 @@ writeMriImageToNifti <- function (image, fileNames, gzipped = FALSE, datatype =
     if (!is(image, "MriImage"))
         report(OL$Error, "The specified image is not an MriImage object")
     
-    description <- "TractoR NIfTI writer v2.2.0"
+    description <- "TractoR NIfTI writer v2.4.0"
     fileFun <- (if (gzipped) gzfile else file)
     
-    if (is.null(datatype))
-    {
-        datatype <- image$getDataType()
-        if (length(datatype) == 0)
-            report(OL$Error, "The data type is not stored with the image; it must be specified")
-    }
-    
-    typeCode <- getNiftiCodeForDataType(datatype)
-    if (is.null(typeCode))
-        report(OL$Error, "No supported NIfTI datatype is appropriate for this file")
+    datatype <- chooseDataTypeForImage(image, "Nifti")
     
     ndims <- image$getDimensionality()
     fullDims <- c(ndims, image$getDimensions(), rep(1,7-ndims))
     fullVoxelDims <- c(-1, abs(image$getVoxelDimensions()), rep(0,7-ndims))
     
     # We default to 10 (mm and s)
-    unitName <- image$getVoxelUnit()
+    unitName <- image$getVoxelUnits()
     unitCode <- as.numeric(.Nifti$units[names(.Nifti$units) %in% unitName])
     if (length(unitCode) == 0)
         unitCode <- 10
     else
         unitCode <- sum(unitCode)
     
-    origin <- (image$getOrigin() - 1) * abs(image$getVoxelDimensions())
-    if (length(origin) > 3)
-        origin <- origin[1:3]
-    else if (length(origin) < 3)
-        origin <- c(origin, rep(0,3-length(origin)))
-    origin <- ifelse(origin < 0, rep(0,3), origin)
-    origin[2:3] <- -origin[2:3]
-    sformRows <- c(-fullVoxelDims[2], 0, 0, origin[1],
-                    0, fullVoxelDims[3], 0, origin[2],
-                    0, 0, fullVoxelDims[4], origin[3])
+    if (image$isReordered())
+    {
+        origin <- (image$getOrigin() - 1) * abs(image$getVoxelDimensions())
+        if (length(origin) > 3)
+            origin <- origin[1:3]
+        else if (length(origin) < 3)
+            origin <- c(origin, rep(0,3-length(origin)))
+        origin <- ifelse(origin < 0, rep(0,3), origin)
+        origin[2:3] <- -origin[2:3]
+        sformRows <- c(-fullVoxelDims[2], 0, 0, origin[1],
+                        0, fullVoxelDims[3], 0, origin[2],
+                        0, 0, fullVoxelDims[4], origin[3])
+        
+        quaternion <- list(q=c(0,0,1,0), offset=origin, handedness=-1)
+    }
+    else
+    {
+        xform <- image$getStoredXformMatrix()
+        sformRows <- c(xform[1,], xform[2,], xform[3,])
+        quaternion <- xformToQuaternion(xform)
+        fullVoxelDims[1] <- quaternion$handedness
+    }
     
     connection <- fileFun(fileNames$headerFile, "w+b")
     
@@ -210,7 +175,7 @@ writeMriImageToNifti <- function (image, fileNames, gzipped = FALSE, datatype =
     writeBin(raw(36), connection)
     writeBin(as.integer(fullDims), connection, size=2)
     writeBin(raw(14), connection)
-    writeBin(as.integer(typeCode), connection, size=2)
+    writeBin(as.integer(datatype$code), connection, size=2)
     writeBin(as.integer(8*datatype$size), connection, size=2)
     writeBin(raw(2), connection)
     writeBin(fullVoxelDims, connection, size=4)
@@ -220,7 +185,8 @@ writeMriImageToNifti <- function (image, fileNames, gzipped = FALSE, datatype =
     
     writeBin(raw(3), connection)
     writeBin(as.integer(unitCode), connection, size=1)
-    writeBin(raw(24), connection)
+    writeBin(as.double(rev(range(image$getData(),na.rm=TRUE))), connection, size=4)
+    writeBin(raw(16), connection)
     writeBin(charToRaw(description), connection, size=1)
     writeBin(raw(24+80-nchar(description)), connection)
     
@@ -229,7 +195,8 @@ writeMriImageToNifti <- function (image, fileNames, gzipped = FALSE, datatype =
         writeBin(as.integer(c(0,0)), connection, size=2)
     else
         writeBin(as.integer(c(2,2)), connection, size=2)
-    writeBin(c(0,1,0,origin), connection, size=4)
+    writeBin(quaternion$q[2:4], connection, size=4)
+    writeBin(quaternion$offset, connection, size=4)
     writeBin(sformRows, connection, size=4)
     
     writeBin(raw(16), connection)
@@ -245,7 +212,7 @@ writeMriImageToNifti <- function (image, fileNames, gzipped = FALSE, datatype =
         connection <- fileFun(fileNames$imageFile, "w+b")
     }
     
-    writeImageData(image, connection)
+    writeImageData(image, connection, datatype$type, datatype$size)
     close(connection)
     
     if (image$isInternal())
